@@ -1,6 +1,6 @@
 /* channel.c
  *
- * $Id: channel.c,v 1.65 2000/01/12 22:46:00 nisse Exp $
+ * $Id: channel.c,v 1.66 2000/01/20 22:58:22 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -346,7 +346,7 @@ void dealloc_channel(struct channel_table *table, int i)
   assert(i >= 0);
   assert( (unsigned) i < table->used_channels);
 
-  verbose("Deallocating local channel %i\n");
+  verbose("Deallocating local channel %i\n", i);
   table->channels[i] = NULL;
   table->in_use[i] = 0;
   
