@@ -1,6 +1,6 @@
 /* channel_commands.h
  *
- * $Id: channel_commands.h,v 1.10 1999/06/22 22:43:18 nisse Exp $
+ * $Id: channel_commands.h,v 1.12 1999/08/18 21:11:24 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -49,9 +49,10 @@
 
 #define NEW_CHANNEL(s, c,r) ((s)->new_channel((s), (c), (r)))
 
-int do_channel_open_command(struct command *s,
+void do_channel_open_command(struct command *s,
 			    struct lsh_object *x,
-			    struct command_continuation *c);
+			    struct command_continuation *c,
+			    struct exception_handler *e);
 
 /* Takes a channel as argument, and returns the same channel or NULL. */
 /* GABA:
@@ -70,9 +71,10 @@ int do_channel_open_command(struct command *s,
 #define FORMAT_CHANNEL_REQUEST(r, c, w) \
 ((r)->format_request((r), (c), (w)))
 
-int do_channel_request_command(struct command *s,
+void do_channel_request_command(struct command *s,
 			       struct lsh_object *x,
-			       struct command_continuation *c);
+			       struct command_continuation *c,
+			       struct exception_handler *e);
 
 /* GABA:
    (class
@@ -87,9 +89,10 @@ int do_channel_request_command(struct command *s,
 #define FORMAT_GLOBAL_REQUEST(r, conn, c) \
 ((r)->format_request((r), (conn), (c)))
 
-int do_channel_global_command(struct command *s,
-			      struct lsh_object *x,
-			      struct command_continuation *c);
+void do_channel_global_command(struct command *s,
+			       struct lsh_object *x,
+			       struct command_continuation *c,
+			       struct exception_handler *e);
 
 struct command *
 make_install_global_request_handler(UINT32 name,
