@@ -1,6 +1,6 @@
 /* lsh_types.h
  *
- * $Id: lsh_types.h,v 1.35 1999/10/13 20:22:29 nisse Exp $ */
+ * $Id: lsh_types.h,v 1.36 2000/01/09 18:35:32 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -28,6 +28,25 @@
 #include "config.h"
 #endif
 
+#ifndef alloca
+# ifdef __GNUC__
+#  define alloca __builtin_alloca
+#  define HAVE_ALLOCA 1
+# else
+#  if defined HAVE_ALLOCA_H || defined _LIBC
+#   include <alloca.h>
+#  else
+#   ifdef _AIX
+ #pragma alloca
+#   else
+#    ifndef alloca
+char *alloca ();
+#    endif
+#   endif
+#  endif
+# endif
+#endif
+
 #if SIZEOF_SHORT >= 4
 # define UINT32 unsigned short
 #elif SIZEOF_INT >= 4
